extern "C" struct hostent * gethostbyname(const char *);

#ifdef __sun__
#include "/usr/ucbinclude/fcntl.h"
#include "b.hh"
extern "C" int gethostname(char*,int);
#endif

#include <stdlib.h>
#include <unistd.h>
#include <iostream.h>
#include <fcntl.h>
#include <string.h> 
#include <errno.h>
#include <ctype.h>

#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/param.h>

#include "debug.hh"

#ifdef __osf__
#include "net.h"
//extern "C" int gethostname(const char *,int);
#endif

#include "list.hh"
#include "sys_socket.hh"



/************************************************************************
**                            SYS_SOCKET                               **
************************************************************************/

SYS_Socket::SYS_Socket()
{
  //cerr << "SYS_Socket\n";
  if((s = socket(AF_INET,SOCK_DGRAM,0)) != 0)
    status = SYS_SOCKET_STATUS_NOT_BINDED;
  else
    status = SYS_SOCKET_STATUS_NO_SOCKET;
}
  
SYS_Socket::~SYS_Socket()
{ 
  if(status != SYS_SOCKET_STATUS_NO_SOCKET)
    shutdown(s,2);
}

int SYS_Socket::Bind(SYS_Address& ssa)
{
  //cerr << "SYS_Socket::Bind()\n";
  if((ssa.Status() == SYS_SOCKET_ADDRESS_STATUS_SET)&&
     (this.Status() != SYS_SOCKET_STATUS_NO_SOCKET))
    if(bind(s,(sockaddr *)&(ssa.sa), sizeof(sockaddr_in)) == 0)	
      status = SYS_SOCKET_STATUS_OK;	
    else
      status = SYS_SOCKET_STATUS_NOT_BINDED;
}  

int SYS_Socket::Status()
{
  return status;
}

/*
int SYS_Socket::FileDescriptor()
{
  return s;
}
*/

/************************************************************************
**                            SYS_SEND_SOCKET                          **
************************************************************************/

SYS_Send_Socket::SYS_Send_Socket()
{
  //cerr << "SYS_Send_Socket protected\n";
  ;
}

/*
SYS_Send_Socket::SYS_Send_Socket(SYS_Address& ssa)
{
  cerr << "SYS_Send_Socket\n";
  if(SYS_Socket::Status() != SYS_SOCKET_STATUS_NO_SOCKET)    
    SYS_Socket::Bind(ssa);
}
*/

SYS_Send_Socket::~SYS_Send_Socket()
{
  ;
}  

int SYS_Send_Socket::SetSendBufferSize(int Size)
{
  int dim = sizeof(int);
  int newSize = 0;
  setsockopt(s, SOL_SOCKET, SO_SNDBUF,(char *) &Size,sizeof(int));
  getsockopt(s, SOL_SOCKET, SO_SNDBUF,(char *) &newSize,&dim);
  if(newSize == Size)
    return 0;
  else
    return -1;
}

int SYS_Send_Socket::GetSendBufferSize()
{
  int dim = sizeof(int);
  int Size = 0;
  if(getsockopt(s, SOL_SOCKET, SO_SNDBUF,(char *) &Size,&dim) < 0)
    return -1;
  else
    return Size;
}


int SYS_Send_Socket::Send(SYS_Address& ssa,void* pData,int len)
{
  if(SYS_Socket::Status() != SYS_SOCKET_STATUS_OK)
    return -1;

  if(ssa.Status() != SYS_SOCKET_ADDRESS_STATUS_SET)
    return -1;

  int  sended = sendto(s,(char *)pData,len,0,
		       (sockaddr *) &(ssa.sa),sizeof(sockaddr_in));

  if(sended < 0)
    {
      cerr << "SYS_Send_Socket::Send(): errno = " << errno << '\n';
      perror("SYS_Send_Socket::Send():");
    }	       

  //cout << "SYS_SOCKET: length of sended data = " << sended << '\n';  
  return sended;
}

#ifdef __osf__
int SYS_Send_Socket::Send(SYS_Address& ssa,void** pDataArray,int* pLengthArray,int elements)
{
  static iovec  iov[32];

  if(SYS_Socket::Status() != SYS_SOCKET_STATUS_OK)
    return -1;

  if(ssa.Status() != SYS_SOCKET_ADDRESS_STATUS_SET)
    return -1;

  if(elements >= 32)
    return -1;

  msghdr  msgh;

  bzero((char*) &msgh,sizeof(msghdr));
  bzero((char*) iov,32*sizeof(iovec));

  msgh.msg_name    =(caddr_t) &(ssa.sa);
  msgh.msg_namelen = sizeof(sockaddr_in);
  msgh.msg_iov     = iov;
  msgh.msg_iovlen  = elements;
  for(unsigned int i=0;i<elements;i++)
    {
      iov[i].iov_base =(caddr_t) pDataArray[i];
      iov[i].iov_len  = pLengthArray[i];
    }

  int sended = sendmsg(s,&msgh,0);
  if(sended < 0)
    perror("SYS_Send_Socket::Send(): sendmsg: ");

  //cout << "SYS_SOCKET: length of sended data = " << sended << '\n';  
  return sended;
}
#endif

/************************************************************************
**                          SYS_RECEIVE_SOCKET                         **
************************************************************************/


#ifdef _DEBUG_
void dbg_what(void* b,int i)
{
  for(int j=0;j<i;j++)
    cerr << "buff " << j+1 << " = " << (char) *((char*)b+j) << '\n';
}
#endif 

SYS_Receive_Socket::SYS_Receive_Socket()
{
  //cerr << "SYS_Receive_Socket protected\n";
  options = 0x0;
  packets2 = 0;
  gpackets = 0;
  //IOHandlerFunction = this->DefaultIOHandler;
}


/*
SYS_Receive_Socket::SYS_Receive_Socket(SYS_Address& ssa)
{
  cerr << "SYS_Receive_Socket\n";
  if(SYS_Socket::Status() != SYS_SOCKET_STATUS_NO_SOCKET)    
    SYS_Socket::Bind(ssa);
  options = 0x0;
  packets2 = 0;
  gpackets = 0;
}
*/

SYS_Receive_Socket::~SYS_Receive_Socket()
{
  cout << "- Total packets received from socket " << s << " is: " << gpackets << '\n';
  ;
}  


int SYS_Receive_Socket::SetOwn()
{
  cerr << "SYS RECEIVE SOCKET: SetOwn() called\n";
  if(SYS_Socket::Status() == SYS_SOCKET_STATUS_NO_SOCKET)
    return -1;

  if(fcntl(s,F_SETOWN,getpid()) < 0)
    {
      cerr << "SYS RECEIVE SOCKET: SetOwn() error\n";
      return -1;
    }
  else
    {
      options |= SYS_RECEIVE_SOCKET_OPTIONS_OWNED;
      //cout << "SetOwn\n";
      return 0;
    }
}


int SYS_Receive_Socket::SetAsyncronousMode()
{
  if(options & SYS_RECEIVE_SOCKET_OPTIONS_OWNED)
    if(fcntl(s,F_SETFL,FNDELAY|FASYNC) < 0)
      {
	cerr << "SYS RECEIVE SOCKET: SetAsyncronousMode() error\n";
	return -1;
      }
    else
      {
	options |= SYS_RECEIVE_SOCKET_OPTIONS_ASYNC;
	//cout << "SetAsync\n";
	return 0;
      }
  else
    return -1;
}


int SYS_Receive_Socket::SetSyncronousMode()
{
#ifdef __osf__
  if(options & SYS_RECEIVE_SOCKET_OPTIONS_OWNED)
    if(fcntl(s,F_SETFL,FSYNC) < 0)
      {
	cerr << "SYS RECEIVE SOCKET: SetSyncronousMode() error\n";
	return -1;
      }
    else 
      {
	options |= SYS_RECEIVE_SOCKET_OPTIONS_SYNC;
	return 0;
      }
  else
    return -1;
#endif
#ifndef __osf__
  return -1;
#endif
}

int SYS_Receive_Socket::HaveData()
{
  /*
  if(SYS_Socket::Status() != SYS_SOCKET_STATUS_OK)
    return -1;

  //cout << "SYS_SOCKET: start recv\n";  
  int readed = recv(s,(char*)buffer,len,MSG_PEEK);
  //cout << "SYS_SOCKET: end recv\n";  
  */
  
  fd_set   rfds;
  timeval  t;
  t.tv_sec = 0;
  t.tv_usec = 0;

  FD_ZERO(&rfds);
  FD_SET(s,&rfds);

  select(5,&rfds,NULL,NULL,&t);

  if(FD_ISSET(s,&rfds))
    {
      //cout << "yes\n";
      return 1;
    }
  else
    {
      //cout << "no\n";
      return 0;
    }
}

int SYS_Receive_Socket::Receive(void* buffer,int len)
{
  if(SYS_Socket::Status() != SYS_SOCKET_STATUS_OK)
    return -1;

  //cout << "SYS_SOCKET: start recv\n";  
  int readed = recv(s,(char*)buffer,len,0);
  //cout << "SYS_SOCKET: end recv\n";  
  //cout << "SYS_SOCKET: length of readed data = " << readed << '\n';  
  
  return readed;
}


int SYS_Receive_Socket::SetReceiveBufferSize(int Size)
{
  int dim = sizeof(int);
  int newSize = 0;
  setsockopt(s, SOL_SOCKET, SO_RCVBUF,(char *) &Size,sizeof(int));
  getsockopt(s, SOL_SOCKET, SO_RCVBUF,(char *) &newSize,&dim);
  if(newSize == Size)
    return 0;
  else
    return -1;
}

int SYS_Receive_Socket::GetReceiveBufferSize()
{
  int dim = sizeof(int);
  int Size = 0;
  if(getsockopt(s, SOL_SOCKET, SO_RCVBUF,(char *) &Size,&dim) < 0)
    return -1;
  else
    return Size;
}

/*
int SYS_Receive_Socket::SetHandlerFunction(void (*fun)(SYS_Receive_Socket*))
{
  IOHandlerFunction = fun;
  return 0;
}
*/

/************************************************************************
**                        SYS_SEND_RECEIVE_SOCKET                      **
************************************************************************/

/*
SYS_Send_Receive_Socket::SYS_Send_Receive_Socket(SYS_Address& ssa)
: SYS_Send_Socket(), SYS_Receive_Socket()
{
  cerr << "SYS_Send_Receive_Socket\n";
  if(SYS_Socket::Status() != SYS_SOCKET_STATUS_NO_SOCKET)    
    SYS_Socket::Bind(ssa);
}
*/

SYS_Send_Receive_Socket::SYS_Send_Receive_Socket()
: SYS_Send_Socket(), SYS_Receive_Socket()
{
  //cerr << "SYS_Send_Receive_Socket\n";
  ;
}

SYS_Send_Receive_Socket::~SYS_Send_Receive_Socket()
{
  ;
}  










